"""
Downloads
=========
Download example datasets from https://github.com/ansys/example-data"""
import os
import urllib.request
import warnings
from ansys.dpf.core.examples.examples import find_files

EXAMPLE_REPO = "https://github.com/ansys/example-data/raw/master/"


def delete_downloads(verbose=True):
    """Delete all downloaded examples to free space or update the files"""
    from ansys.dpf.core import LOCAL_DOWNLOADED_EXAMPLES_PATH, examples

    not_to_remove = [
        getattr(examples.examples, item)
        for item in dir(examples.examples)
        if not item.startswith("_")
        and not item.endswith("_")
        and isinstance(getattr(examples.examples, item), str)
    ]
    not_to_remove.extend(
        [
            os.path.join(os.path.dirname(examples.__file__), "__init__.py"),
            os.path.join(os.path.dirname(examples.__file__), "downloads.py"),
            os.path.join(os.path.dirname(examples.__file__), "examples.py"),
        ]
    )
    for root, dirs, files in os.walk(LOCAL_DOWNLOADED_EXAMPLES_PATH, topdown=False):
        if root not in not_to_remove:
            for name in files:
                if not os.path.join(root, name) in not_to_remove:
                    try:
                        os.remove(os.path.join(root, name))
                        if verbose:
                            print(f"deleting {os.path.join(root, name)}")
                    except Exception as e:
                        warnings.warn(
                            f"couldn't delete {os.path.join(root, name)} with error:\n {e.args}"
                        )
    for root, dirs, files in os.walk(LOCAL_DOWNLOADED_EXAMPLES_PATH, topdown=False):
        if len(dirs) == 0 and len(files) == 0:
            try:
                os.rmdir(root)
                if verbose:
                    print(f"deleting {root}")
            except Exception as e:
                warnings.warn(f"couldn't delete {root} with error:\n {e.args}")


def _get_file_url(directory, filename):
    return EXAMPLE_REPO + "/".join([directory, filename])


def _retrieve_file(url, filename, directory):
    """Download a file from a url"""
    from ansys.dpf.core import LOCAL_DOWNLOADED_EXAMPLES_PATH

    # First check if file has already been downloaded
    local_path = os.path.join(LOCAL_DOWNLOADED_EXAMPLES_PATH, directory, os.path.basename(filename))
    local_path_no_zip = local_path.replace(".zip", "")
    if os.path.isfile(local_path_no_zip) or os.path.isdir(local_path_no_zip):
        return local_path_no_zip

    # grab the correct url retriever
    urlretrieve = urllib.request.urlretrieve

    dirpath = os.path.dirname(local_path)
    if not os.path.isdir(dirpath):
        os.makedirs(dirpath, exist_ok=True)

    # Perform download
    _, resp = urlretrieve(url, local_path)
    return local_path


def _download_file(directory, filename, should_upload: bool, server, return_local_path):
    url = _get_file_url(directory, filename)
    local_path = _retrieve_file(url, filename, directory)
    return find_files(local_path, should_upload, server, return_local_path)


###############################################################################
# front-facing functions


def download_transient_result(
    should_upload: bool = True, server=None, return_local_path=False
) -> str:
    """Download an example transient result file and return the download path
    available server side.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> path = examples.download_transient_result()
    >>> path
    'C:/Users/user/AppData/local/temp/transient.rst'

    """
    return _download_file(
        "result_files/transient", "transient.rst", should_upload, server, return_local_path
    )


def download_all_kinds_of_complexity(
    should_upload: bool = True, server=None, return_local_path=False
) -> str:
    """Download an example static result and return the download path
    available server side.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> path = examples.download_all_kinds_of_complexity()
    >>> path
    'C:/Users/user/AppData/local/temp/allKindOfComplexity.rst'

    """
    return _download_file(
        "result_files/testing", "allKindOfComplexity.rst", should_upload, server, return_local_path
    )


def download_all_kinds_of_complexity_modal(
    should_upload: bool = True, server=None, return_local_path=False
) -> str:
    """Download an example result file from a static modal analysis and
    return the download path available server side.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> path = examples.download_all_kinds_of_complexity_modal()
    >>> path
    'C:/Users/user/AppData/local/temp/modal_allKindOfComplexity.rst'

    """
    return _download_file(
        "result_files/testing",
        "modal_allKindOfComplexity.rst",
        should_upload,
        server,
        return_local_path,
    )


def download_pontoon(should_upload: bool = True, server=None, return_local_path=False) -> str:
    """Download an example result file from a static modal analsys and
    return the download path available server side.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> path = examples.download_pontoon()
    >>> path
    'C:/Users/user/AppData/local/temp/pontoon.rst'

    """
    return _download_file(
        "result_files/docs", "pontoon.rst", should_upload, server, return_local_path
    )


def download_multi_harmonic_result(
    should_upload: bool = True, server=None, return_local_path=False
) -> str:
    """Download an example multi-harmonic result file and return the
    download path available server side.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> path = examples.download_multi_harmonic_result()
    >>> path
    'C:/Users/user/AppData/local/temp/file_harmonic_5rpms.rst'
    """
    return _download_file(
        "result_files/harmonic", "file_harmonic_5rpms.rst", should_upload, server, return_local_path
    )


def download_multi_stage_cyclic_result(
    should_upload: bool = True, server=None, return_local_path=False
) -> str:
    """Download an example multi-stage result file and return the
    download path available server side.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> path = examples.download_multi_stage_cyclic_result()
    >>> path
    'C:/Users/user/AppData/local/temp/multistage.rst'

    """
    return _download_file(
        "result_files/multistage", "multistage.rst", should_upload, server, return_local_path
    )


def download_sub_file(should_upload: bool = True, server=None, return_local_path=False) -> str:
    """Download an example .sub result file containing matrices and return the
    download path available server side.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> path = examples.download_sub_file()
    >>> path
    'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\sub\\cp56.sub'

    """
    return _download_file("result_files/sub", "cp56.sub", should_upload, server, return_local_path)


def download_msup_files_to_dict(
    should_upload: bool = True, server=None, return_local_path=False
) -> dict:
    """Download all the files necessary for a msup expansion and return the
    download paths available server side into a dictionary extension->path.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    dict[str:str]
        Path to the example files.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> paths = examples.download_msup_files_to_dict()
    >>> paths
    {'rfrq': 'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\msup\\file.rfrq',
     'mode': 'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\msup\\file.mode',
     'rst': 'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\msup\\file.rst'} # noqa: E501

    """
    return {
        "rfrq": _download_file(
            "result_files/msup", "file.rfrq", should_upload, server, return_local_path
        ),
        "mode": _download_file(
            "result_files/msup", "file.mode", should_upload, server, return_local_path
        ),
        "rst": _download_file(
            "result_files/msup", "file.rst", should_upload, server, return_local_path
        ),
    }


def download_distributed_files(
    should_upload: bool = True, server=None, return_local_path=False
) -> dict:
    """Download distributed rst files and return the
    download paths into a dictionary domain id->path.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    dict[int:str]
        Path to the example files.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> paths = examples.download_distributed_files()
    >>> paths
    {0: 'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\distributed\\file0.rst',
     1: 'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\distributed\\file1.rst'} # noqa: E501

    """
    return {
        0: _download_file(
            "result_files/distributed", "file0.rst", should_upload, server, return_local_path
        ),
        1: _download_file(
            "result_files/distributed", "file1.rst", should_upload, server, return_local_path
        ),
    }


def download_fluent_multi_species(
    should_upload: bool = True, server=None, return_local_path=False
) -> dict:
    """Download the cas and dat file of a fluent analysis with multiple species
    and return the download paths into a dictionary extension->path.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    dict[str:str]
        Path to the example files.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> paths = examples.download_fluent_multi_species()
    >>> paths
    {'cas': 'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-multi_species\\FFF.cas.h5',
     'dat': 'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-multi_species\\FFF.dat.h5'} # noqa: E501

    """
    return {
        "cas": _download_file(
            "result_files/fluent-multi_species",
            "FFF.cas.h5",
            should_upload,
            server,
            return_local_path,
        ),
        "dat": _download_file(
            "result_files/fluent-multi_species",
            "FFF.dat.h5",
            should_upload,
            server,
            return_local_path,
        ),
    }


def download_fluent_multi_phase(
    should_upload: bool = True, server=None, return_local_path=False
) -> dict:
    """Download the cas and dat file of a fluent analysis with multiple phases
    and return the download paths into a dictionary extension->path.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    dict[str:str]
        Path to the example files.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> paths = examples.download_fluent_multi_phase()
    >>> paths
    {'cas': 'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-multi_phase\\fluentMultiphase.cas.h5',
     'dat': 'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-multi_phase\\fluentMultiphase.dat.h5'} # noqa: E501

    """
    return {
        "cas": _download_file(
            "result_files/fluent-multi_phase",
            "fluentMultiphase.cas.h5",
            should_upload,
            server,
            return_local_path,
        ),
        "dat": _download_file(
            "result_files/fluent-multi_phase",
            "fluentMultiphase.dat.h5",
            should_upload,
            server,
            return_local_path,
        ),
    }


def download_extrapolation_3d_result(
    should_upload: bool = True, server=None, return_local_path=False
) -> dict:
    """Download example static results of reference and integrated points
    for extrapolation of 3d-element and return the dictionary of 2 download paths.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    dict
        containing path to the example file of ref and path to the example
        file of integrated points.

    Examples
    --------
    Download 2 example result files and return the dictionary containing 2 files

    >>> from ansys.dpf.core import examples
    >>> path_dict = examples.download_extrapolation_ref_result()
    >>> path_dict
    {
        'file_ref': 'C:/Users/user/AppData/local/temp/file_ref.rst',
        'file_integrated': 'C:/Users/user/AppData/local/temp/file.rst'
    }

    """
    path_dict = {
        "file_ref": _download_file(
            "result_files/extrapolate", "file_ref.rst", should_upload, server, return_local_path
        ),
        "file_integrated": _download_file(
            "result_files/extrapolate", "file.rst", should_upload, server, return_local_path
        ),
    }

    return path_dict


def download_extrapolation_2d_result(
    should_upload: bool = True, server=None, return_local_path=False
) -> dict:
    """Download example static results of reference and integrated points
    for extrapolation of 2d-element and return the dictionary of 2 download paths.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    dict
        Contains path to the example file of ref and path to the example
        file of integrated points.

    Examples
    --------
    Download 2 example result files and return the dictionary containing 2 files

    >>> from ansys.dpf.core import examples
    >>> path_dict = examples.download_extrapolation_ref_result()
    >>> path_dict
    {
        'file_ref': 'C:/Users/user/AppData/local/temp/extrapolate_2d_ref.rst',
        'file_integrated': 'C:/Users/user/AppData/local/temp/extrapolate_2d.rst'
    }

    """
    path_dict = {
        "file_ref": _download_file(
            "result_files/extrapolate",
            "extrapolate_2d_ref.rst",
            should_upload,
            server,
            return_local_path,
        ),
        "file_integrated": _download_file(
            "result_files/extrapolate",
            "extrapolate_2d.rst",
            should_upload,
            server,
            return_local_path,
        ),
    }

    return path_dict


def download_hemisphere(should_upload: bool = True, server=None, return_local_path=False) -> str:
    """Download an example result file from a static analysis and
    return the download path available server side.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> path = examples.download_hemisphere()
    >>> path
    'C:/Users/user/AppData/local/temp/hemisphere.rst'

    """
    return _download_file(
        "result_files/hemisphere", "hemisphere.rst", should_upload, server, return_local_path
    )


def download_example_asme_result(
    should_upload: bool = True, server=None, return_local_path=False
) -> str:
    """Download an example result file from a static analysis and
    return the download path available server side.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file
    >>> from ansys.dpf.core import examples
    >>> path = examples.download_example_asme_result()
    >>> path
    'C:/Users/user/AppData/local/temp/asme_example.rst'
    """
    return _download_file(
        "result_files/postprocessing", "asme_example.rst", should_upload, server, return_local_path
    )


def download_crankshaft(should_upload: bool = True, server=None, return_local_path=False) -> str:
    """Download the result file of an example of a crankshaft
    under load and return the download path available server side.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> path = examples.download_crankshaft()
    >>> path
    'C:/Users/user/AppData/local/temp/crankshaft.rst'

    """
    return _download_file(
        "result_files/crankshaft", "crankshaft.rst", should_upload, server, return_local_path
    )


def download_piston_rod(should_upload: bool = True, server=None, return_local_path=False) -> str:
    """Download the result file of an example of a piston rod
    under load and return the download path available server side.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> path = examples.download_piston_rod()
    >>> path
    'C:/Users/user/AppData/local/temp/piston_rod.rst'

    """
    return _download_file(
        "result_files/piston_rod", "piston_rod.rst", should_upload, server, return_local_path
    )


def download_d3plot_beam(should_upload: bool = True, server=None, return_local_path=False) -> list:
    """Download the result file of an example of a d3plot file with beam elements and return the
    download paths available on the server side.
    If the server is remote (or doesn't share the memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded to the server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the files.

    >>> from ansys.dpf.core import examples
    >>> paths = examples.download_d3plot_beam()
    >>> paths
    ['C:/Users/user/AppData/local/temp/d3plot',
     'C:/Users/user/AppData/local/temp/d3plot01',
     'C:/Users/user/AppData/local/temp/d3plot02'
     'C:/Users/user/AppData/local/temp/file.actunits']

    """
    return [
        _download_file(
            "result_files/d3plot_beam", "d3plot", should_upload, server, return_local_path
        ),
        _download_file(
            "result_files/d3plot_beam", "d3plot01", should_upload, server, return_local_path
        ),
        _download_file(
            "result_files/d3plot_beam", "d3plot02", should_upload, server, return_local_path
        ),
        _download_file(
            "result_files/d3plot_beam", "file.actunits", should_upload, server, return_local_path
        ),
    ]


def download_binout_matsum(should_upload: bool = True, server=None, return_local_path=False) -> str:
    """Download the result file of an example of a binout file with matsum branch and return the
    download path available on the server side.
    If the server is remote (or doesn't share the memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded to the server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file.

    >>> from ansys.dpf.core import examples
    >>> path = examples.download_binout_matsum()
    >>> path
    'C:/Users/user/AppData/local/temp/binout_matsum'

    """
    return _download_file(
        "result_files/binout", "binout_matsum", should_upload, server, return_local_path
    )


def download_binout_glstat(should_upload: bool = True, server=None, return_local_path=False) -> str:
    """Download the result file of an example of a binout file with glstat branch and return the
    download path available on the server side.
    If the server is remote (or doesn't share the memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded to the server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file.

    >>> from ansys.dpf.core import examples
    >>> path = examples.download_binout_glstat()
    >>> path
    'C:/Users/user/AppData/local/temp/binout_glstat'

    """
    return _download_file(
        "result_files/binout", "binout_glstat", should_upload, server, return_local_path
    )


def download_cycles_to_failure(
    should_upload: bool = True, server=None, return_local_path=False
) -> str:
    """Download an example result file from a cyclic analysis and
    return the download path.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> path = examples.download_cycles_to_failure()
    >>> path
    'C:/Users/user/AppData/local/temp/cycles_to_failure.rst'

    """
    return _download_file(
        "result_files/cyclic", "cyclic_to_failure.rst", should_upload, server, return_local_path
    )


def download_modal_frame(should_upload: bool = True, server=None, return_local_path=False) -> str:
    """Download an example result file from a modal analysis on a frame and
    return the download path.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> path = examples.download_modal_frame()

    """
    return _download_file(
        "result_files/modal", "frame.rst", should_upload, server, return_local_path
    )


def download_harmonic_clamped_pipe(
    should_upload: bool = True, server=None, return_local_path=False
) -> str:
    """Download an example result file from a harmonic analysis on a clamped pipe and
    return the download path.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> path = examples.download_modal_frame()

    """
    return _download_file(
        "result_files/harmonic", "clamped_pipe.rst", should_upload, server, return_local_path
    )


def download_modal_cyclic(should_upload: bool = True, server=None, return_local_path=False) -> str:
    """Download an example result file from a cyclic modal analysis and
    return the download path.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    str
        Path to the example file.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> path = examples.download_modal_cyclic()

    """
    return _download_file(
        "result_files/cyclic", "modal_cyclic.rst", should_upload, server, return_local_path
    )


def download_fluent_axial_comp(
    should_upload: bool = True, server=None, return_local_path=False
) -> dict:
    """Download the flprj, cas and dat files of a fluent analysis of an axial compressor sector
    and return the download paths into a dictionary extension->path.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    dict[str:str]
        Path to the example files.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> paths = examples.download_fluent_axial_comp()
    >>> paths
    {'flprj': 'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-axial_comp\\axial_comp_reduced.flprj',
     'cas': [
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-axial_comp\\axial_comp-1-01438.cas.h5',
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-axial_comp\\axial_comp-1-01439.cas.h5',
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-axial_comp\\axial_comp-1-01440.cas.h5',
     ],
     'dat': [
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-axial_comp\\axial_comp-1-01438.dat.h5',
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-axial_comp\\axial_comp-1-01439.dat.h5',
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-axial_comp\\axial_comp-1-01440.dat.h5',
     ]} # noqa: E501

    """
    return {
        "flprj": _download_file(
            "result_files/fluent-axial_comp",
            "axial_comp_reduced.flprj",
            should_upload,
            server,
            return_local_path,
        ),
        "cas": [
            _download_file(
                "result_files/fluent-axial_comp",
                "axial_comp-1-01438.cas.h5",
                should_upload,
                server,
                return_local_path,
            ),
            _download_file(
                "result_files/fluent-axial_comp",
                "axial_comp-1-01439.cas.h5",
                should_upload,
                server,
                return_local_path,
            ),
            _download_file(
                "result_files/fluent-axial_comp",
                "axial_comp-1-01440.cas.h5",
                should_upload,
                server,
                return_local_path,
            ),
        ],
        "dat": [
            _download_file(
                "result_files/fluent-axial_comp",
                "axial_comp-1-01438.dat.h5",
                should_upload,
                server,
                return_local_path,
            ),
            _download_file(
                "result_files/fluent-axial_comp",
                "axial_comp-1-01439.dat.h5",
                should_upload,
                server,
                return_local_path,
            ),
            _download_file(
                "result_files/fluent-axial_comp",
                "axial_comp-1-01440.dat.h5",
                should_upload,
                server,
                return_local_path,
            ),
        ],
    }


def download_fluent_mixing_elbow_steady_state(
    should_upload: bool = True, server=None, return_local_path=False
) -> dict:
    """Download the flprj, cas and dat files of a steady-state fluent analysis of a mixing elbow
    and return the download paths into a dictionary extension->path.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    dict[str:str]
        Path to the example files.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> paths = examples.download_fluent_mixing_elbow_steady_state()
    >>> paths
    {'flprj': 'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-mixing_elbow_steady-state\\elbow.flprj',
     'cas': [
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-mixing_elbow_steady-state\\elbow-2.cas.h5',
     ],
     'dat': [
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-mixing_elbow_steady-state\\elbow-2-00005.dat.h5',
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-mixing_elbow_steady-state\\elbow-2-00010.dat.h5',
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-mixing_elbow_steady-state\\elbow-2-00015.dat.h5',
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-mixing_elbow_steady-state\\elbow-2-00020.dat.h5',
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-mixing_elbow_steady-state\\elbow-2-00025.dat.h5',
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-mixing_elbow_steady-state\\elbow-2-00030.dat.h5',
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-mixing_elbow_steady-state\\elbow-2-00035.dat.h5',
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-mixing_elbow_steady-state\\elbow-2-00040.dat.h5',
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-mixing_elbow_steady-state\\elbow-2-00045.dat.h5',
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-mixing_elbow_steady-state\\elbow-2-00050.dat.h5',
     ]} # noqa: E501

    """
    return {
        "flprj": _download_file(
            "result_files/fluent-mixing_elbow_steady-state",
            "elbow.flprj",
            should_upload,
            server,
            return_local_path,
        ),
        "cas": [
            _download_file(
                "result_files/fluent-mixing_elbow_steady-state",
                "elbow-2.cas.h5",
                should_upload,
                server,
                return_local_path,
            ),
        ],
        "dat": [
            _download_file(
                "result_files/fluent-mixing_elbow_steady-state",
                "elbow-2-00005.dat.h5",
                should_upload,
                server,
                return_local_path,
            ),
            _download_file(
                "result_files/fluent-mixing_elbow_steady-state",
                "elbow-2-00010.dat.h5",
                should_upload,
                server,
                return_local_path,
            ),
            _download_file(
                "result_files/fluent-mixing_elbow_steady-state",
                "elbow-2-00015.dat.h5",
                should_upload,
                server,
                return_local_path,
            ),
            _download_file(
                "result_files/fluent-mixing_elbow_steady-state",
                "elbow-2-00020.dat.h5",
                should_upload,
                server,
                return_local_path,
            ),
            _download_file(
                "result_files/fluent-mixing_elbow_steady-state",
                "elbow-2-00025.dat.h5",
                should_upload,
                server,
                return_local_path,
            ),
            _download_file(
                "result_files/fluent-mixing_elbow_steady-state",
                "elbow-2-00030.dat.h5",
                should_upload,
                server,
                return_local_path,
            ),
            _download_file(
                "result_files/fluent-mixing_elbow_steady-state",
                "elbow-2-00035.dat.h5",
                should_upload,
                server,
                return_local_path,
            ),
            _download_file(
                "result_files/fluent-mixing_elbow_steady-state",
                "elbow-2-00040.dat.h5",
                should_upload,
                server,
                return_local_path,
            ),
            _download_file(
                "result_files/fluent-mixing_elbow_steady-state",
                "elbow-2-00045.dat.h5",
                should_upload,
                server,
                return_local_path,
            ),
            _download_file(
                "result_files/fluent-mixing_elbow_steady-state",
                "elbow-2-00050.dat.h5",
                should_upload,
                server,
                return_local_path,
            ),
        ],
    }


def download_fluent_mixing_elbow_transient(
    should_upload: bool = True, server=None, return_local_path=False
) -> dict:
    """Download the flprj, cas and dat files of a transient fluent analysis of a mixing elbow
    and return the download paths into a dictionary extension->path.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    dict[str:str]
        Path to the example files.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> paths = examples.download_fluent_mixing_elbow_transient()
    >>> paths
    {'flprj': 'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-mixing_elbow_transient\\elbow.flprj',
     'cas': [
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-mixing_elbow_transient\\elbow-2.cas.h5',
     ],
     'dat': [
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-mixing_elbow_transient\\elbow-2-00001.dat.h5',
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-mixing_elbow_transient\\elbow-2-00002.dat.h5',
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-mixing_elbow_transient\\elbow-2-00003.dat.h5',
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-mixing_elbow_transient\\elbow-2-00004.dat.h5',
       'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\fluent-mixing_elbow_transient\\elbow-2-00005.dat.h5',
     ]} # noqa: E501

    """
    return {
        "flprj": _download_file(
            "result_files/fluent-mixing_elbow_transient",
            "elbow.flprj",
            should_upload,
            server,
            return_local_path,
        ),
        "cas": [
            _download_file(
                "result_files/fluent-mixing_elbow_transient",
                "elbow-2.cas.h5",
                should_upload,
                server,
                return_local_path,
            ),
        ],
        "dat": [
            _download_file(
                "result_files/fluent-mixing_elbow_transient",
                "elbow-2-00001.dat.h5",
                should_upload,
                server,
                return_local_path,
            ),
            _download_file(
                "result_files/fluent-mixing_elbow_transient",
                "elbow-2-00002.dat.h5",
                should_upload,
                server,
                return_local_path,
            ),
            _download_file(
                "result_files/fluent-mixing_elbow_transient",
                "elbow-2-00003.dat.h5",
                should_upload,
                server,
                return_local_path,
            ),
            _download_file(
                "result_files/fluent-mixing_elbow_transient",
                "elbow-2-00004.dat.h5",
                should_upload,
                server,
                return_local_path,
            ),
            _download_file(
                "result_files/fluent-mixing_elbow_transient",
                "elbow-2-00005.dat.h5",
                should_upload,
                server,
                return_local_path,
            ),
        ],
    }


def download_cfx_heating_coil(
    should_upload: bool = True, server=None, return_local_path=False
) -> dict:
    """Download the flprj, cas and dat files of a CFX analysis of a heating coil
    and return the download paths into a dictionary extension->path.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    dict[str:str]
        Path to the example files.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> paths = examples.download_cfx_heating_coil()
    >>> paths
    {'cas': 'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\cfx-heating_coil\\HeatingCoil.res',
     'dat': 'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\cfx-heating_coil\\HeatingCoil.res'} # noqa: E501

    """
    file = _download_file(
        "result_files/cfx-heating_coil",
        "HeatingCoil.res",
        should_upload,
        server,
        return_local_path,
    )
    return {"cas": file, "dat": file}


def download_cfx_mixing_elbow(
    should_upload: bool = True, server=None, return_local_path=False
) -> dict:
    """Download the flprj, cas and dat files of a CFX analysis of a mixing elbow
    and return the download paths into a dictionary extension->path.
    If the server is remote (or doesn't share memory), the file is uploaded or made available
    on the server side.

    Examples files are downloaded to a persistent cache to avoid
    re-downloading the same file twice.

    Parameters
    ----------
    should_upload : bool, optional (default True)
        Whether the file should be uploaded server side when the server is remote.
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.
    return_local_path: bool, optional
        If ``True``, the local path is returned as is, without uploading, nor searching
        for mounted volumes.

    Returns
    -------
    dict[str:str]
        Path to the example files.

    Examples
    --------
    Download an example result file and return the path of the file

    >>> from ansys.dpf.core import examples
    >>> paths = examples.download_cfx_mixing_elbow()
    >>> paths
    {'cas': 'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\cfx-mixing_elbow\\InjectMixer.res',
     'dat': 'C:\\Users\\user\\AppData\\Local\\ansys-dpf-core\\ansys-dpf-core\\examples\\cfx-mixing_elbow\\InjectMixer.res'} # noqa: E501

    """
    file = _download_file(
        "result_files/cfx-mixing_elbow",
        "InjectMixer.res",
        should_upload,
        server,
        return_local_path,
    )
    return {"cas": file, "dat": file}



"""
cms_dst_table_provider
======================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class cms_dst_table_provider(Operator):
    """Read CST table from a subfile.

    Parameters
    ----------
    data_sources : DataSources
        Data_sources (must contain at least one
        subfile).


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.result.cms_dst_table_provider()

    >>> # Make input connections
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.result.cms_dst_table_provider(
    ...     data_sources=my_data_sources,
    ... )

    >>> # Get output data
    >>> result_dst_table = op.outputs.dst_table()
    """

    def __init__(self, data_sources=None, config=None, server=None):
        super().__init__(name="cms_dst_table_provider", config=config, server=server)
        self._inputs = InputsCmsDstTableProvider(self)
        self._outputs = OutputsCmsDstTableProvider(self)
        if data_sources is not None:
            self.inputs.data_sources.connect(data_sources)

    @staticmethod
    def _spec():
        description = """Read CST table from a subfile."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                4: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=False,
                    document="""Data_sources (must contain at least one
        subfile).""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="dst_table",
                    type_names=["property_field"],
                    optional=False,
                    document="""Returns integer values of the dst table""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="cms_dst_table_provider", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsCmsDstTableProvider
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsCmsDstTableProvider
        """
        return super().outputs


class InputsCmsDstTableProvider(_Inputs):
    """Intermediate class used to connect user inputs to
    cms_dst_table_provider operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.cms_dst_table_provider()
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    """

    def __init__(self, op: Operator):
        super().__init__(cms_dst_table_provider._spec().inputs, op)
        self._data_sources = Input(
            cms_dst_table_provider._spec().input_pin(4), 4, op, -1
        )
        self._inputs.append(self._data_sources)

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator.

        Data_sources (must contain at least one
        subfile).

        Parameters
        ----------
        my_data_sources : DataSources

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.cms_dst_table_provider()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> # or
        >>> op.inputs.data_sources(my_data_sources)
        """
        return self._data_sources


class OutputsCmsDstTableProvider(_Outputs):
    """Intermediate class used to get outputs from
    cms_dst_table_provider operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.cms_dst_table_provider()
    >>> # Connect inputs : op.inputs. ...
    >>> result_dst_table = op.outputs.dst_table()
    """

    def __init__(self, op: Operator):
        super().__init__(cms_dst_table_provider._spec().outputs, op)
        self._dst_table = Output(cms_dst_table_provider._spec().output_pin(0), 0, op)
        self._outputs.append(self._dst_table)

    @property
    def dst_table(self):
        """Allows to get dst_table output of the operator

        Returns
        ----------
        my_dst_table : PropertyField

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.cms_dst_table_provider()
        >>> # Connect inputs : op.inputs. ...
        >>> result_dst_table = op.outputs.dst_table()
        """  # noqa: E501
        return self._dst_table



"""
split_in_for_each_range
=======================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class split_in_for_each_range(Operator):
    """Split a scoping into several pieces so you can iterate it with a
    for_each loop.

    Parameters
    ----------
    iterable : optional
        Iterable that can be combined with the one
        currently generated.
    operator_to_iterate : Operator
        Operator that must be reconnected with the
        range values.
    pin_index : int
    scoping : Scoping
    chunk_size : int


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.split_in_for_each_range()

    >>> # Make input connections
    >>> my_iterable = dpf.()
    >>> op.inputs.iterable.connect(my_iterable)
    >>> my_operator_to_iterate = dpf.Operator()
    >>> op.inputs.operator_to_iterate.connect(my_operator_to_iterate)
    >>> my_pin_index = int()
    >>> op.inputs.pin_index.connect(my_pin_index)
    >>> my_scoping = dpf.Scoping()
    >>> op.inputs.scoping.connect(my_scoping)
    >>> my_chunk_size = int()
    >>> op.inputs.chunk_size.connect(my_chunk_size)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.split_in_for_each_range(
    ...     iterable=my_iterable,
    ...     operator_to_iterate=my_operator_to_iterate,
    ...     pin_index=my_pin_index,
    ...     scoping=my_scoping,
    ...     chunk_size=my_chunk_size,
    ... )

    >>> # Get output data
    >>> result_output = op.outputs.output()
    """

    def __init__(
        self,
        iterable=None,
        operator_to_iterate=None,
        pin_index=None,
        scoping=None,
        chunk_size=None,
        config=None,
        server=None,
    ):
        super().__init__(name="chunk_in_for_each_range", config=config, server=server)
        self._inputs = InputsSplitInForEachRange(self)
        self._outputs = OutputsSplitInForEachRange(self)
        if iterable is not None:
            self.inputs.iterable.connect(iterable)
        if operator_to_iterate is not None:
            self.inputs.operator_to_iterate.connect(operator_to_iterate)
        if pin_index is not None:
            self.inputs.pin_index.connect(pin_index)
        if scoping is not None:
            self.inputs.scoping.connect(scoping)
        if chunk_size is not None:
            self.inputs.chunk_size.connect(chunk_size)

    @staticmethod
    def _spec():
        description = """Split a scoping into several pieces so you can iterate it with a
            for_each loop."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="iterable",
                    type_names=["any"],
                    optional=True,
                    document="""Iterable that can be combined with the one
        currently generated.""",
                ),
                1: PinSpecification(
                    name="operator_to_iterate",
                    type_names=["operator"],
                    optional=False,
                    document="""Operator that must be reconnected with the
        range values.""",
                ),
                2: PinSpecification(
                    name="pin_index",
                    type_names=["int32"],
                    optional=False,
                    document="""""",
                ),
                3: PinSpecification(
                    name="scoping",
                    type_names=["scoping"],
                    optional=False,
                    document="""""",
                ),
                4: PinSpecification(
                    name="chunk_size",
                    type_names=["int32"],
                    optional=False,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="output",
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="chunk_in_for_each_range", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsSplitInForEachRange
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsSplitInForEachRange
        """
        return super().outputs


class InputsSplitInForEachRange(_Inputs):
    """Intermediate class used to connect user inputs to
    split_in_for_each_range operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.split_in_for_each_range()
    >>> my_iterable = dpf.()
    >>> op.inputs.iterable.connect(my_iterable)
    >>> my_operator_to_iterate = dpf.Operator()
    >>> op.inputs.operator_to_iterate.connect(my_operator_to_iterate)
    >>> my_pin_index = int()
    >>> op.inputs.pin_index.connect(my_pin_index)
    >>> my_scoping = dpf.Scoping()
    >>> op.inputs.scoping.connect(my_scoping)
    >>> my_chunk_size = int()
    >>> op.inputs.chunk_size.connect(my_chunk_size)
    """

    def __init__(self, op: Operator):
        super().__init__(split_in_for_each_range._spec().inputs, op)
        self._iterable = Input(split_in_for_each_range._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._iterable)
        self._operator_to_iterate = Input(
            split_in_for_each_range._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._operator_to_iterate)
        self._pin_index = Input(split_in_for_each_range._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._pin_index)
        self._scoping = Input(split_in_for_each_range._spec().input_pin(3), 3, op, -1)
        self._inputs.append(self._scoping)
        self._chunk_size = Input(
            split_in_for_each_range._spec().input_pin(4), 4, op, -1
        )
        self._inputs.append(self._chunk_size)

    @property
    def iterable(self):
        """Allows to connect iterable input to the operator.

        Iterable that can be combined with the one
        currently generated.

        Parameters
        ----------
        my_iterable :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.split_in_for_each_range()
        >>> op.inputs.iterable.connect(my_iterable)
        >>> # or
        >>> op.inputs.iterable(my_iterable)
        """
        return self._iterable

    @property
    def operator_to_iterate(self):
        """Allows to connect operator_to_iterate input to the operator.

        Operator that must be reconnected with the
        range values.

        Parameters
        ----------
        my_operator_to_iterate : Operator

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.split_in_for_each_range()
        >>> op.inputs.operator_to_iterate.connect(my_operator_to_iterate)
        >>> # or
        >>> op.inputs.operator_to_iterate(my_operator_to_iterate)
        """
        return self._operator_to_iterate

    @property
    def pin_index(self):
        """Allows to connect pin_index input to the operator.

        Parameters
        ----------
        my_pin_index : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.split_in_for_each_range()
        >>> op.inputs.pin_index.connect(my_pin_index)
        >>> # or
        >>> op.inputs.pin_index(my_pin_index)
        """
        return self._pin_index

    @property
    def scoping(self):
        """Allows to connect scoping input to the operator.

        Parameters
        ----------
        my_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.split_in_for_each_range()
        >>> op.inputs.scoping.connect(my_scoping)
        >>> # or
        >>> op.inputs.scoping(my_scoping)
        """
        return self._scoping

    @property
    def chunk_size(self):
        """Allows to connect chunk_size input to the operator.

        Parameters
        ----------
        my_chunk_size : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.split_in_for_each_range()
        >>> op.inputs.chunk_size.connect(my_chunk_size)
        >>> # or
        >>> op.inputs.chunk_size(my_chunk_size)
        """
        return self._chunk_size


class OutputsSplitInForEachRange(_Outputs):
    """Intermediate class used to get outputs from
    split_in_for_each_range operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.split_in_for_each_range()
    >>> # Connect inputs : op.inputs. ...
    >>> result_output = op.outputs.output()
    """

    def __init__(self, op: Operator):
        super().__init__(split_in_for_each_range._spec().outputs, op)
        self._output = Output(split_in_for_each_range._spec().output_pin(0), 0, op)
        self._outputs.append(self._output)

    @property
    def output(self):
        """Allows to get output output of the operator

        Returns
        ----------
        my_output :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.split_in_for_each_range()
        >>> # Connect inputs : op.inputs. ...
        >>> result_output = op.outputs.output()
        """  # noqa: E501
        return self._output



"""
grpc_start_server
=================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class grpc_start_server(Operator):
    """Starts a dpf's grpc server (if local) or connect to one and keep it
    waiting for requests in a streams.

    Parameters
    ----------
    ip : str, optional
        If no ip address is put, the local ip address
        is taken
    port : str or int, optional
        If no port is put, port 50052 is taken
    starting_option : int, optional
        Default is 1 that starts server in new
        thread. with 0, this thread will be
        waiting for grpc calls and will not
        be usable for anything else. with 2,
        it the server will be started in a
        new process.
    should_start_server : bool, optional
        If true, the server is assumed to be local
        and is started. if false, only a
        client (able to send grpc calls) will
        be started
    data_sources : DataSources, optional
        A data sources with result key 'grpc' and
        file path port:ip can be used instead
        of the input port and ip.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.server.grpc_start_server()

    >>> # Make input connections
    >>> my_ip = str()
    >>> op.inputs.ip.connect(my_ip)
    >>> my_port = str()
    >>> op.inputs.port.connect(my_port)
    >>> my_starting_option = int()
    >>> op.inputs.starting_option.connect(my_starting_option)
    >>> my_should_start_server = bool()
    >>> op.inputs.should_start_server.connect(my_should_start_server)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.server.grpc_start_server(
    ...     ip=my_ip,
    ...     port=my_port,
    ...     starting_option=my_starting_option,
    ...     should_start_server=my_should_start_server,
    ...     data_sources=my_data_sources,
    ... )

    >>> # Get output data
    >>> result_grpc_streams = op.outputs.grpc_streams()
    """

    def __init__(
        self,
        ip=None,
        port=None,
        starting_option=None,
        should_start_server=None,
        data_sources=None,
        config=None,
        server=None,
    ):
        super().__init__(name="grpc::stream_provider", config=config, server=server)
        self._inputs = InputsGrpcStartServer(self)
        self._outputs = OutputsGrpcStartServer(self)
        if ip is not None:
            self.inputs.ip.connect(ip)
        if port is not None:
            self.inputs.port.connect(port)
        if starting_option is not None:
            self.inputs.starting_option.connect(starting_option)
        if should_start_server is not None:
            self.inputs.should_start_server.connect(should_start_server)
        if data_sources is not None:
            self.inputs.data_sources.connect(data_sources)

    @staticmethod
    def _spec():
        description = """Starts a dpf's grpc server (if local) or connect to one and keep it
            waiting for requests in a streams."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="ip",
                    type_names=["string"],
                    optional=True,
                    document="""If no ip address is put, the local ip address
        is taken""",
                ),
                1: PinSpecification(
                    name="port",
                    type_names=["string", "int32"],
                    optional=True,
                    document="""If no port is put, port 50052 is taken""",
                ),
                2: PinSpecification(
                    name="starting_option",
                    type_names=["int32"],
                    optional=True,
                    document="""Default is 1 that starts server in new
        thread. with 0, this thread will be
        waiting for grpc calls and will not
        be usable for anything else. with 2,
        it the server will be started in a
        new process.""",
                ),
                3: PinSpecification(
                    name="should_start_server",
                    type_names=["bool"],
                    optional=True,
                    document="""If true, the server is assumed to be local
        and is started. if false, only a
        client (able to send grpc calls) will
        be started""",
                ),
                4: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=True,
                    document="""A data sources with result key 'grpc' and
        file path port:ip can be used instead
        of the input port and ip.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="grpc_streams",
                    type_names=["streams_container"],
                    optional=True,
                    document="""Dpf streams handling the server, if the
        server is started in this thread,
        then nothing is added in output""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="grpc::stream_provider", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsGrpcStartServer
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsGrpcStartServer
        """
        return super().outputs


class InputsGrpcStartServer(_Inputs):
    """Intermediate class used to connect user inputs to
    grpc_start_server operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.server.grpc_start_server()
    >>> my_ip = str()
    >>> op.inputs.ip.connect(my_ip)
    >>> my_port = str()
    >>> op.inputs.port.connect(my_port)
    >>> my_starting_option = int()
    >>> op.inputs.starting_option.connect(my_starting_option)
    >>> my_should_start_server = bool()
    >>> op.inputs.should_start_server.connect(my_should_start_server)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    """

    def __init__(self, op: Operator):
        super().__init__(grpc_start_server._spec().inputs, op)
        self._ip = Input(grpc_start_server._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._ip)
        self._port = Input(grpc_start_server._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._port)
        self._starting_option = Input(grpc_start_server._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._starting_option)
        self._should_start_server = Input(
            grpc_start_server._spec().input_pin(3), 3, op, -1
        )
        self._inputs.append(self._should_start_server)
        self._data_sources = Input(grpc_start_server._spec().input_pin(4), 4, op, -1)
        self._inputs.append(self._data_sources)

    @property
    def ip(self):
        """Allows to connect ip input to the operator.

        If no ip address is put, the local ip address
        is taken

        Parameters
        ----------
        my_ip : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.server.grpc_start_server()
        >>> op.inputs.ip.connect(my_ip)
        >>> # or
        >>> op.inputs.ip(my_ip)
        """
        return self._ip

    @property
    def port(self):
        """Allows to connect port input to the operator.

        If no port is put, port 50052 is taken

        Parameters
        ----------
        my_port : str or int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.server.grpc_start_server()
        >>> op.inputs.port.connect(my_port)
        >>> # or
        >>> op.inputs.port(my_port)
        """
        return self._port

    @property
    def starting_option(self):
        """Allows to connect starting_option input to the operator.

        Default is 1 that starts server in new
        thread. with 0, this thread will be
        waiting for grpc calls and will not
        be usable for anything else. with 2,
        it the server will be started in a
        new process.

        Parameters
        ----------
        my_starting_option : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.server.grpc_start_server()
        >>> op.inputs.starting_option.connect(my_starting_option)
        >>> # or
        >>> op.inputs.starting_option(my_starting_option)
        """
        return self._starting_option

    @property
    def should_start_server(self):
        """Allows to connect should_start_server input to the operator.

        If true, the server is assumed to be local
        and is started. if false, only a
        client (able to send grpc calls) will
        be started

        Parameters
        ----------
        my_should_start_server : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.server.grpc_start_server()
        >>> op.inputs.should_start_server.connect(my_should_start_server)
        >>> # or
        >>> op.inputs.should_start_server(my_should_start_server)
        """
        return self._should_start_server

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator.

        A data sources with result key 'grpc' and
        file path port:ip can be used instead
        of the input port and ip.

        Parameters
        ----------
        my_data_sources : DataSources

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.server.grpc_start_server()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> # or
        >>> op.inputs.data_sources(my_data_sources)
        """
        return self._data_sources


class OutputsGrpcStartServer(_Outputs):
    """Intermediate class used to get outputs from
    grpc_start_server operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.server.grpc_start_server()
    >>> # Connect inputs : op.inputs. ...
    >>> result_grpc_streams = op.outputs.grpc_streams()
    """

    def __init__(self, op: Operator):
        super().__init__(grpc_start_server._spec().outputs, op)
        self._grpc_streams = Output(grpc_start_server._spec().output_pin(0), 0, op)
        self._outputs.append(self._grpc_streams)

    @property
    def grpc_streams(self):
        """Allows to get grpc_streams output of the operator

        Returns
        ----------
        my_grpc_streams : StreamsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.server.grpc_start_server()
        >>> # Connect inputs : op.inputs. ...
        >>> result_grpc_streams = op.outputs.grpc_streams()
        """  # noqa: E501
        return self._grpc_streams



"""
merge_meshes_containers
=======================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class merge_meshes_containers(Operator):
    """Assembles a set of meshes containers into a unique one.

    Parameters
    ----------
    meshes_containers1 : MeshesContainer
        A vector of meshes containers to merge or
        meshes containers from pin 0 to ...
    meshes_containers2 : MeshesContainer
        A vector of meshes containers to merge or
        meshes containers from pin 0 to ...


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.merge_meshes_containers()

    >>> # Make input connections
    >>> my_meshes_containers1 = dpf.MeshesContainer()
    >>> op.inputs.meshes_containers1.connect(my_meshes_containers1)
    >>> my_meshes_containers2 = dpf.MeshesContainer()
    >>> op.inputs.meshes_containers2.connect(my_meshes_containers2)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.merge_meshes_containers(
    ...     meshes_containers1=my_meshes_containers1,
    ...     meshes_containers2=my_meshes_containers2,
    ... )

    >>> # Get output data
    >>> result_merged_meshes_container = op.outputs.merged_meshes_container()
    """

    def __init__(
        self, meshes_containers1=None, meshes_containers2=None, config=None, server=None
    ):
        super().__init__(name="merge::meshes_container", config=config, server=server)
        self._inputs = InputsMergeMeshesContainers(self)
        self._outputs = OutputsMergeMeshesContainers(self)
        if meshes_containers1 is not None:
            self.inputs.meshes_containers1.connect(meshes_containers1)
        if meshes_containers2 is not None:
            self.inputs.meshes_containers2.connect(meshes_containers2)

    @staticmethod
    def _spec():
        description = """Assembles a set of meshes containers into a unique one."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="meshes_containers",
                    type_names=["meshes_container"],
                    optional=False,
                    document="""A vector of meshes containers to merge or
        meshes containers from pin 0 to ...""",
                ),
                1: PinSpecification(
                    name="meshes_containers",
                    type_names=["meshes_container"],
                    optional=False,
                    document="""A vector of meshes containers to merge or
        meshes containers from pin 0 to ...""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="merged_meshes_container",
                    type_names=["meshes_container"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="merge::meshes_container", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMergeMeshesContainers
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsMergeMeshesContainers
        """
        return super().outputs


class InputsMergeMeshesContainers(_Inputs):
    """Intermediate class used to connect user inputs to
    merge_meshes_containers operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.merge_meshes_containers()
    >>> my_meshes_containers1 = dpf.MeshesContainer()
    >>> op.inputs.meshes_containers1.connect(my_meshes_containers1)
    >>> my_meshes_containers2 = dpf.MeshesContainer()
    >>> op.inputs.meshes_containers2.connect(my_meshes_containers2)
    """

    def __init__(self, op: Operator):
        super().__init__(merge_meshes_containers._spec().inputs, op)
        self._meshes_containers1 = Input(
            merge_meshes_containers._spec().input_pin(0), 0, op, 0
        )
        self._inputs.append(self._meshes_containers1)
        self._meshes_containers2 = Input(
            merge_meshes_containers._spec().input_pin(1), 1, op, 1
        )
        self._inputs.append(self._meshes_containers2)

    @property
    def meshes_containers1(self):
        """Allows to connect meshes_containers1 input to the operator.

        A vector of meshes containers to merge or
        meshes containers from pin 0 to ...

        Parameters
        ----------
        my_meshes_containers1 : MeshesContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.merge_meshes_containers()
        >>> op.inputs.meshes_containers1.connect(my_meshes_containers1)
        >>> # or
        >>> op.inputs.meshes_containers1(my_meshes_containers1)
        """
        return self._meshes_containers1

    @property
    def meshes_containers2(self):
        """Allows to connect meshes_containers2 input to the operator.

        A vector of meshes containers to merge or
        meshes containers from pin 0 to ...

        Parameters
        ----------
        my_meshes_containers2 : MeshesContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.merge_meshes_containers()
        >>> op.inputs.meshes_containers2.connect(my_meshes_containers2)
        >>> # or
        >>> op.inputs.meshes_containers2(my_meshes_containers2)
        """
        return self._meshes_containers2


class OutputsMergeMeshesContainers(_Outputs):
    """Intermediate class used to get outputs from
    merge_meshes_containers operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.merge_meshes_containers()
    >>> # Connect inputs : op.inputs. ...
    >>> result_merged_meshes_container = op.outputs.merged_meshes_container()
    """

    def __init__(self, op: Operator):
        super().__init__(merge_meshes_containers._spec().outputs, op)
        self._merged_meshes_container = Output(
            merge_meshes_containers._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._merged_meshes_container)

    @property
    def merged_meshes_container(self):
        """Allows to get merged_meshes_container output of the operator

        Returns
        ----------
        my_merged_meshes_container : MeshesContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.merge_meshes_containers()
        >>> # Connect inputs : op.inputs. ...
        >>> result_merged_meshes_container = op.outputs.merged_meshes_container()
        """  # noqa: E501
        return self._merged_meshes_container



"""
settings
========
Customize the behavior of the module.
"""

from ansys.dpf.core.misc import module_exists
from ansys.dpf.core import misc
from ansys.dpf.core.server import set_server_configuration  # noqa: F401
from ansys.dpf.core.server_context import set_default_server_context  # noqa: F401
from ansys.dpf.core.server_factory import ServerConfig  # noqa: F401
from ansys.dpf.core import core


def disable_off_screen_rendering() -> None:
    """No pop up windows appears to plot data with ``matplotlib`` or ``pyvista``"""
    # enable matplotlib off_screen plotting to avoid test interruption
    if module_exists("matplotlib"):
        import matplotlib as mpl

        mpl.use("Agg")

    # enable off_screen plotting to avoid test interruption
    if module_exists("pyvista"):
        import pyvista as pv

        pv.OFF_SCREEN = True


def set_default_pyvista_config():
    # Configure PyVista's ``rcParams`` for dpf
    if module_exists("pyvista"):
        import pyvista as pv

        pv.global_theme.interactive = True
        pv.global_theme.cmap = "jet"
        pv.global_theme.font.family = "courier"
        pv.global_theme.title = "DPF"


def bypass_pv_opengl_osmesa_crash():
    if module_exists("pyvista"):
        import pyvista as pv

        pv.global_theme.lighting = False


def disable_interpreter_properties_evaluation() -> bool:
    """If ``jedi`` module is installed (autocompletion module for most of IDEs), disables the
    property evaluation when tab key is pressed.

    To use in Jupyter Notebook if autocompletion becomes slow.

    Returns
    -------
    bool
        Whether disabling the capability has been possible.
    """
    if module_exists("jedi"):
        import jedi

        jedi.Interpreter._allow_descriptor_getattr_default = False
        return True
    return False


def set_upload_chunk_size(num_bytes=misc.DEFAULT_FILE_CHUNK_SIZE) -> None:
    misc.DEFAULT_FILE_CHUNK_SIZE = num_bytes


def set_dynamic_available_results_capability(value) -> None:
    """Disables the evaluation of the available results and
    the dynamic creation of the results properties when a ''Model'' is created.

    Parameters
    ----------
    value :  bool
        With ''True'', models will dynamically generate results properties

    Examples
    --------

    >>> from ansys.dpf import core as dpf
    >>> dpf.settings.set_dynamic_available_results_capability(False)
    >>> dpf.settings.set_dynamic_available_results_capability(True)

    """
    misc.DYNAMIC_RESULTS = value


def _forward_to_gate():
    from ansys.dpf.gate import settings
    from ansys.dpf.core.misc import DEFAULT_FILE_CHUNK_SIZE
    from ansys.dpf.core.common import _common_progress_bar, _progress_bar_is_available

    settings.forward_settings(
        DEFAULT_FILE_CHUNK_SIZE,
        _common_progress_bar if _progress_bar_is_available() else None,
    )


def get_runtime_client_config(server=None):
    """Get the runtime configuration information of Ans.Dpf.GrpcClient
    binary.

    Parameters
    ----------
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.

    Notes
    -----
    Available from 4.0 server version. Can only be used for
    a gRPC communication protocol using DPF CLayer.

    Returns
    -------
    runtime_config : RuntimeClientConfig
        RuntimeClientConfig object that can be used to interact
        with Ans.Dpf.GrpcClient configuration.

    """
    base = core.BaseService(server, load_operators=False)
    return base.get_runtime_client_config()


def get_runtime_core_config(server=None):
    """Get the runtime configuration information of Ans.Dpf.GrpcClient
    binary.

    Parameters
    ----------
    server : server.DPFServer, optional
        Server with channel connected to the remote or local instance. When
        ``None``, attempts to use the global server.

    Notes
    -----
    Available from 4.0 server version.

    Returns
    -------
    core_config : RuntimeCoreConfig
        RuntimeCoreConfig object that can be used to interact
        with DataProcessingCore configuration.

    """
    base = core.BaseService(server, load_operators=False)
    return base.get_runtime_core_config()



"""
.. _ref_errors:

Errors
======

.. autoexception:: DpfVersionNotSupported
   :members:

.. autoexception:: DPFServerNullObject
   :members:
"""

from functools import wraps
from ansys.dpf.gate.errors import (  # noqa: F401
    DPFServerException,
    DPFServerNullObject,
    DpfVersionNotSupported,
)  # noqa: F401

_COMPLEX_PLOTTING_ERROR_MSG = """
Complex fields cannot be plotted. Use operators to get the amplitude
or the result at a defined sweeping phase before plotting.
"""

_FIELD_CONTAINER_PLOTTING_MSG = """"
This fields_container contains multiple fields.  Only one time-step
result can be plotted at a time. Extract a field with
``fields_container[index]``.
"""


class DpfValueError(ValueError):
    """Error raised when a specific DPF error value must be defined."""

    def __init__(self, msg="A value that has been set leads to incorrect DPF behavior."):
        ValueError.__init__(self, msg)


class InvalidTypeError(ValueError):
    """Error raised when a parameter has the wrong type."""

    def __init__(self, data_type, parameter_name):
        msg = (
            "A " + data_type + " must be used for the following parameter: " + parameter_name + "."
        )
        ValueError.__init__(self, msg)


class LocationError(ValueError):
    """Error raised when using an invalid location."""

    def __init__(self, msg="Invalid location"):
        ValueError.__init__(self, msg)


class ComplexPlottingError(ValueError):
    """Error raised when attempting to plot a field with complex data."""

    def __init__(self, msg=_COMPLEX_PLOTTING_ERROR_MSG):
        ValueError.__init__(self, msg)


class FieldContainerPlottingError(ValueError):
    """Error raised when attempting to plot a fields_container containing
    multiple fields."""

    def __init__(self, msg=_FIELD_CONTAINER_PLOTTING_MSG):
        ValueError.__init__(self, msg)


class InvalidANSYSVersionError(RuntimeError):
    """Error raised when the Ansys version is invalid."""

    def __init__(self, msg=""):
        RuntimeError.__init__(self, msg)


class InvalidPortError(OSError):
    """Error raised when used an invalid port when starting DPF."""

    def __init__(self, msg=""):
        OSError.__init__(self, msg)


class ServerTypeError(NotImplementedError):
    """Error raised when using a functionality unavailable for this server type"""

    pass


def protect_source_op_not_found(func):
    """Capture DPF's Server exceptions when a source operator is not found
    and return a more succinct error message.
    """

    @wraps(func)
    def wrapper(*args, **kwargs):
        """Capture DPF's Server exceptions."""
        try:
            out = func(*args, **kwargs)
        except DPFServerException as error:
            details = str(error)
            if "source operator not found" in details:
                return None
            raise DPFServerException(details)

        return out

    return wrapper



"""
Operator Configuration
======================
"""

import functools
import warnings
import traceback

from ansys.dpf.core import server as server_module
from ansys.dpf.gate import (
    operator_config_capi,
    operator_config_grpcapi,
    operator_config_abstract_api,
)
from ansys.dpf.core.operator_specification import Specification


class Config:
    """Represents an operator's configuration.

    You can use configurations to choose how an operator will run.
    This is an advanced feature for deep customization.
    The different options can change the way loops are done.
    They can also change whether the operator needs to perform checks on the input.

    Parameters
    ----------
    operator_name : str, optional
        Name of the operator. The default is ``None``.
    config : str, optional
        Name of the configuration. The default is ``None``.
    server : str, optional
        Server with the channel connected to the remote or local instance. The default is
        ``None``, in which case an attempt is made to use the global server.

    Examples
    --------
    Create an operator configuration, instantiate the operator and
    run it with the configuration.

    >>> from ansys.dpf import core as dpf
    >>> config_add = dpf.Config("add")
    >>> config_add.set_work_by_index_option(True)
    >>> op = dpf.operators.math.add(config=config_add)

    Modify the copy of an operator's configuration and set it as current config
    of the operator.

    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.math.add()
    >>> config_add = op.config
    >>> config_add.set_work_by_index_option(True)
    >>> op.config = config_add

    """

    def __init__(self, operator_name=None, config=None, server=None, spec=None):
        # step 1: get server
        self._server = server_module.get_or_create_server(server)

        # step 2: get api
        self._api_instance = None  # see _api property

        # step3: init environment
        self._api.init_operator_config_environment(self)  # creates stub when gRPC

        # step4: if object exists: take instance, else create it
        if config:
            self._internal_obj = config
        else:
            if self._server.has_client():
                if operator_name:
                    self._internal_obj = self._api.operator_config_default_new_on_client(
                        self._server.client, operator_name
                    )
                else:
                    self._internal_obj = self._api.operator_config_empty_new_on_client(
                        self._server.client
                    )
            else:
                if operator_name:
                    self._internal_obj = self._api.operator_config_default_new(operator_name)
                else:
                    self._internal_obj = self._api.operator_config_empty_new()

        self._operator_name = operator_name
        self._spec_instance = spec
        self._config_help_instance = None

        opt = self.options
        for name in opt:
            bound_method = self.config_option_value.__get__(self, self.__class__)
            method2 = functools.partial(bound_method, config_name=name)
            setattr(self, "get_" + name + "_option", method2)

            bound_method = self.__set_config_option__.__get__(self, self.__class__)
            method2 = functools.partial(bound_method, config_name=name)
            setattr(self, "set_" + name + "_option", method2)

    @property
    def _api(self) -> operator_config_abstract_api.OperatorConfigAbstractAPI:
        if self._api_instance is None:
            self._api_instance = self._server.get_api_for_type(
                capi=operator_config_capi.OperatorConfigCAPI,
                grpcapi=operator_config_grpcapi.OperatorConfigGRPCAPI,
            )
        return self._api_instance

    @property
    def _spec(self):
        if self._spec_instance is None and self._operator_name is not None:
            self._spec_instance = Specification(self._operator_name, server=self._server)
        return self._spec_instance

    @property
    def _config_help(self):
        if self._spec:
            return self._spec.config_specification

    @property
    def options(self):
        """Retrieve a list of configuration options and their values.

        Returns
        -------
        list
            List of configuration options and their values.
        """
        options = {}
        num_options = self._api.operator_config_get_num_config(self)
        for i in range(num_options):
            options[
                self._api.operator_config_get_config_option_name(self, i)
            ] = self._api.operator_config_get_config_option_printable_value(self, i)
        return options

    def __set_config_option__(self, config_value, config_name):
        """Change the value of a configuration option.

        Parameters
        ----------
        config_value : bool, int, float
            Value to give to a configuration option.
        config_name : str
            Name of the configuration option.
        """
        if isinstance(config_value, bool):
            self._api.operator_config_set_bool(self, config_name, config_value)
        elif isinstance(config_value, int):
            self._api.operator_config_set_int(self, config_name, config_value)
        elif isinstance(config_value, float):
            self._api.operator_config_set_double(self, config_name, config_value)
        else:
            raise TypeError("str, int, float are the accepted types for configuration options.")

    def set_config_option(self, config_name, config_value):
        """Change the value of a configuration option.

        Parameters
        ----------
        config_value : bool, int, float
            Value to give to a configuration option.
        config_name : str
            Name of the configuration option.

        Examples
        --------
        Modify the copy of an operator's configuration and set it as current config
        of the operator.

        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.add()
        >>> config_add = op.config
        >>> config_add.set_config_option(config_name="work_by_index", config_value=True)
        >>> op.config = config_add

        """
        return self.__set_config_option__(config_value, config_name)

    def config_option_value(self, config_name):
        """Retrieve the value for a configuration option.

        Parameters
        ----------
        config_name : str
            Name of the configuration option.

        Returns
        ----------
        str
            Value for the configuration option.
        """
        opt = self.options
        if config_name in opt:
            return opt[config_name]
        else:
            raise KeyError(f"{config_name} option doesn't exist.")

    def __try_get_option__(self, config_name):
        if self._config_help:
            if config_name in self._config_help:
                return self._config_help[config_name]
        return None

    def config_option_documentation(self, config_name):
        """Retrieve the documentation for a configuration option.

        Parameters
        ----------
        config_name : str
            Name of the configuration option.

        Returns
        ----------
        str
           Documentation for the configuration option.
        """
        option = self.__try_get_option__(config_name)
        if option:
            return option.document
        return ""

    def config_option_accepted_types(self, config_name):
        """Retrieve accepted types for a configuration option.

        Parameters
        ----------
        config_name : str
            Name of the configuration option.

        Returns
        ----------
        list, str
            One or more accepted types for the configuration option.
        """
        option = self.__try_get_option__(config_name)
        if option:
            return option.type_names

    def config_option_default_value(self, config_name):
        """Retrieve the default value for a configuration option.

        Parameters
        ----------
        config_name : str
            Name of the configuration option.

        Returns
        ----------
        str
            Default value for the configuration option.
        """
        option = self.__try_get_option__(config_name)
        if option:
            return option.default_value_str
        return ""

    @property
    def available_config_options(self):
        """Available configuration options for the operator.

        Returns
        ----------
        list, str
           One or more available configuration options for the operator.
        """
        opt = self.options
        if opt:
            return [key for key in opt]
        return []

    def __str__(self):
        """Describe the entity.

        Returns
        -------
        str
            Description of the entity.
        """
        from ansys.dpf.core.core import _description

        return _description(self._internal_obj, self._server)

    def __del__(self):
        try:
            self._deleter_func[0](self._deleter_func[1](self))
        except:
            warnings.warn(traceback.format_exc())



"""
beam_properties
===============
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class beam_properties(Operator):
    """Reads the beam's properties from the result files contained in the
    streams or data sources.

    Parameters
    ----------
    streams : StreamsContainer, optional
        Result file container allowed to be kept open
        to cache data.
    data_sources : DataSources
        Result file path container, used if no
        streams are set.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.mesh.beam_properties()

    >>> # Make input connections
    >>> my_streams = dpf.StreamsContainer()
    >>> op.inputs.streams.connect(my_streams)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.mesh.beam_properties(
    ...     streams=my_streams,
    ...     data_sources=my_data_sources,
    ... )

    >>> # Get output data
    >>> result_mesh_out = op.outputs.mesh_out()
    >>> result_field_type_section_id = op.outputs.field_type_section_id()
    >>> result_field_area = op.outputs.field_area()
    >>> result_field_moment_inertia = op.outputs.field_moment_inertia()
    >>> result_field_geometry = op.outputs.field_geometry()
    >>> result_field_young_modulus = op.outputs.field_young_modulus()
    >>> result_field_poisson_ratio = op.outputs.field_poisson_ratio()
    >>> result_field_shear_modulus = op.outputs.field_shear_modulus()
    >>> result_field_beam_length = op.outputs.field_beam_length()
    >>> result_field_torsion_constant = op.outputs.field_torsion_constant()
    >>> result_field_warping_constant = op.outputs.field_warping_constant()
    """

    def __init__(self, streams=None, data_sources=None, config=None, server=None):
        super().__init__(name="beam_properties", config=config, server=server)
        self._inputs = InputsBeamProperties(self)
        self._outputs = OutputsBeamProperties(self)
        if streams is not None:
            self.inputs.streams.connect(streams)
        if data_sources is not None:
            self.inputs.data_sources.connect(data_sources)

    @staticmethod
    def _spec():
        description = """Reads the beam's properties from the result files contained in the
            streams or data sources."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                3: PinSpecification(
                    name="streams",
                    type_names=["streams_container"],
                    optional=True,
                    document="""Result file container allowed to be kept open
        to cache data.""",
                ),
                4: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=False,
                    document="""Result file path container, used if no
        streams are set.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="mesh_out",
                    type_names=["abstract_meshed_region"],
                    optional=False,
                    document="""This mesh updates a new map containing a
        field of the beam's properties if
        there is at least one beam in mesh.""",
                ),
                1: PinSpecification(
                    name="field_type_section_id",
                    type_names=["field"],
                    optional=False,
                    document="""This field contains the section id of beams.
        1:rec; 3:csolid, 4:ctube, 5:chan,
        6:z, 7:l, 8:i, 9:t, 11:hats, 12:hrec.""",
                ),
                2: PinSpecification(
                    name="field_area",
                    type_names=["field"],
                    optional=False,
                    document="""This field contains the area of beams.""",
                ),
                3: PinSpecification(
                    name="field_moment_inertia",
                    type_names=["field"],
                    optional=False,
                    document="""This field contains the inertia moment of
        beams. iyy, iyz, izz.""",
                ),
                4: PinSpecification(
                    name="field_geometry",
                    type_names=["field"],
                    optional=False,
                    document="""This field contains the geometry of beams.
        rec:b,h. csolid:ri. ctube:ri, re.
        chan:w1,w2,w3,t1,t2,t3.
        z:w1,w2,w3,t1,t2,t3. l:w1,w2,t1,t2.
        i:w1,w2,w3,t1,t2,t3. t:w1,w2,t1,t2.
        hats: w1,w2,w3,w4,t1,t2,t3,t4.
        hrec:w1,w2,t1,t2,t3,t4.""",
                ),
                5: PinSpecification(
                    name="field_young_modulus",
                    type_names=["field"],
                    optional=False,
                    document="""This field contains the young's modulus of
        beams.""",
                ),
                6: PinSpecification(
                    name="field_poisson_ratio",
                    type_names=["field"],
                    optional=False,
                    document="""This field contains the poisson's ratio of
        beams.""",
                ),
                7: PinSpecification(
                    name="field_shear_modulus",
                    type_names=["field"],
                    optional=False,
                    document="""This field contains the shear modulus of
        beams.""",
                ),
                8: PinSpecification(
                    name="field_beam_length",
                    type_names=["field"],
                    optional=False,
                    document="""This field contains the length of beams.""",
                ),
                9: PinSpecification(
                    name="field_torsion_constant",
                    type_names=["field"],
                    optional=False,
                    document="""This field contains the torsion constant of
        beams.""",
                ),
                10: PinSpecification(
                    name="field_warping_constant",
                    type_names=["field"],
                    optional=False,
                    document="""This field contains the warping constant of
        beams.""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="beam_properties", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsBeamProperties
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsBeamProperties
        """
        return super().outputs


class InputsBeamProperties(_Inputs):
    """Intermediate class used to connect user inputs to
    beam_properties operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mesh.beam_properties()
    >>> my_streams = dpf.StreamsContainer()
    >>> op.inputs.streams.connect(my_streams)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    """

    def __init__(self, op: Operator):
        super().__init__(beam_properties._spec().inputs, op)
        self._streams = Input(beam_properties._spec().input_pin(3), 3, op, -1)
        self._inputs.append(self._streams)
        self._data_sources = Input(beam_properties._spec().input_pin(4), 4, op, -1)
        self._inputs.append(self._data_sources)

    @property
    def streams(self):
        """Allows to connect streams input to the operator.

        Result file container allowed to be kept open
        to cache data.

        Parameters
        ----------
        my_streams : StreamsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.beam_properties()
        >>> op.inputs.streams.connect(my_streams)
        >>> # or
        >>> op.inputs.streams(my_streams)
        """
        return self._streams

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator.

        Result file path container, used if no
        streams are set.

        Parameters
        ----------
        my_data_sources : DataSources

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.beam_properties()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> # or
        >>> op.inputs.data_sources(my_data_sources)
        """
        return self._data_sources


class OutputsBeamProperties(_Outputs):
    """Intermediate class used to get outputs from
    beam_properties operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mesh.beam_properties()
    >>> # Connect inputs : op.inputs. ...
    >>> result_mesh_out = op.outputs.mesh_out()
    >>> result_field_type_section_id = op.outputs.field_type_section_id()
    >>> result_field_area = op.outputs.field_area()
    >>> result_field_moment_inertia = op.outputs.field_moment_inertia()
    >>> result_field_geometry = op.outputs.field_geometry()
    >>> result_field_young_modulus = op.outputs.field_young_modulus()
    >>> result_field_poisson_ratio = op.outputs.field_poisson_ratio()
    >>> result_field_shear_modulus = op.outputs.field_shear_modulus()
    >>> result_field_beam_length = op.outputs.field_beam_length()
    >>> result_field_torsion_constant = op.outputs.field_torsion_constant()
    >>> result_field_warping_constant = op.outputs.field_warping_constant()
    """

    def __init__(self, op: Operator):
        super().__init__(beam_properties._spec().outputs, op)
        self._mesh_out = Output(beam_properties._spec().output_pin(0), 0, op)
        self._outputs.append(self._mesh_out)
        self._field_type_section_id = Output(
            beam_properties._spec().output_pin(1), 1, op
        )
        self._outputs.append(self._field_type_section_id)
        self._field_area = Output(beam_properties._spec().output_pin(2), 2, op)
        self._outputs.append(self._field_area)
        self._field_moment_inertia = Output(
            beam_properties._spec().output_pin(3), 3, op
        )
        self._outputs.append(self._field_moment_inertia)
        self._field_geometry = Output(beam_properties._spec().output_pin(4), 4, op)
        self._outputs.append(self._field_geometry)
        self._field_young_modulus = Output(beam_properties._spec().output_pin(5), 5, op)
        self._outputs.append(self._field_young_modulus)
        self._field_poisson_ratio = Output(beam_properties._spec().output_pin(6), 6, op)
        self._outputs.append(self._field_poisson_ratio)
        self._field_shear_modulus = Output(beam_properties._spec().output_pin(7), 7, op)
        self._outputs.append(self._field_shear_modulus)
        self._field_beam_length = Output(beam_properties._spec().output_pin(8), 8, op)
        self._outputs.append(self._field_beam_length)
        self._field_torsion_constant = Output(
            beam_properties._spec().output_pin(9), 9, op
        )
        self._outputs.append(self._field_torsion_constant)
        self._field_warping_constant = Output(
            beam_properties._spec().output_pin(10), 10, op
        )
        self._outputs.append(self._field_warping_constant)

    @property
    def mesh_out(self):
        """Allows to get mesh_out output of the operator

        Returns
        ----------
        my_mesh_out : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.beam_properties()
        >>> # Connect inputs : op.inputs. ...
        >>> result_mesh_out = op.outputs.mesh_out()
        """  # noqa: E501
        return self._mesh_out

    @property
    def field_type_section_id(self):
        """Allows to get field_type_section_id output of the operator

        Returns
        ----------
        my_field_type_section_id : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.beam_properties()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field_type_section_id = op.outputs.field_type_section_id()
        """  # noqa: E501
        return self._field_type_section_id

    @property
    def field_area(self):
        """Allows to get field_area output of the operator

        Returns
        ----------
        my_field_area : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.beam_properties()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field_area = op.outputs.field_area()
        """  # noqa: E501
        return self._field_area

    @property
    def field_moment_inertia(self):
        """Allows to get field_moment_inertia output of the operator

        Returns
        ----------
        my_field_moment_inertia : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.beam_properties()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field_moment_inertia = op.outputs.field_moment_inertia()
        """  # noqa: E501
        return self._field_moment_inertia

    @property
    def field_geometry(self):
        """Allows to get field_geometry output of the operator

        Returns
        ----------
        my_field_geometry : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.beam_properties()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field_geometry = op.outputs.field_geometry()
        """  # noqa: E501
        return self._field_geometry

    @property
    def field_young_modulus(self):
        """Allows to get field_young_modulus output of the operator

        Returns
        ----------
        my_field_young_modulus : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.beam_properties()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field_young_modulus = op.outputs.field_young_modulus()
        """  # noqa: E501
        return self._field_young_modulus

    @property
    def field_poisson_ratio(self):
        """Allows to get field_poisson_ratio output of the operator

        Returns
        ----------
        my_field_poisson_ratio : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.beam_properties()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field_poisson_ratio = op.outputs.field_poisson_ratio()
        """  # noqa: E501
        return self._field_poisson_ratio

    @property
    def field_shear_modulus(self):
        """Allows to get field_shear_modulus output of the operator

        Returns
        ----------
        my_field_shear_modulus : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.beam_properties()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field_shear_modulus = op.outputs.field_shear_modulus()
        """  # noqa: E501
        return self._field_shear_modulus

    @property
    def field_beam_length(self):
        """Allows to get field_beam_length output of the operator

        Returns
        ----------
        my_field_beam_length : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.beam_properties()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field_beam_length = op.outputs.field_beam_length()
        """  # noqa: E501
        return self._field_beam_length

    @property
    def field_torsion_constant(self):
        """Allows to get field_torsion_constant output of the operator

        Returns
        ----------
        my_field_torsion_constant : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.beam_properties()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field_torsion_constant = op.outputs.field_torsion_constant()
        """  # noqa: E501
        return self._field_torsion_constant

    @property
    def field_warping_constant(self):
        """Allows to get field_warping_constant output of the operator

        Returns
        ----------
        my_field_warping_constant : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.beam_properties()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field_warping_constant = op.outputs.field_warping_constant()
        """  # noqa: E501
        return self._field_warping_constant



"""
ints_to_scoping
===============
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class ints_to_scoping(Operator):
    """take a int or a vector of int and transform it in a one entity field
    of location 'numeric'.

    Parameters
    ----------
    int_or_vector_int : int
        Int or vector of int
    location : str, optional


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.ints_to_scoping()

    >>> # Make input connections
    >>> my_int_or_vector_int = int()
    >>> op.inputs.int_or_vector_int.connect(my_int_or_vector_int)
    >>> my_location = str()
    >>> op.inputs.location.connect(my_location)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.ints_to_scoping(
    ...     int_or_vector_int=my_int_or_vector_int,
    ...     location=my_location,
    ... )

    >>> # Get output data
    >>> result_scoping = op.outputs.scoping()
    """

    def __init__(self, int_or_vector_int=None, location=None, config=None, server=None):
        super().__init__(name="scopingify", config=config, server=server)
        self._inputs = InputsIntsToScoping(self)
        self._outputs = OutputsIntsToScoping(self)
        if int_or_vector_int is not None:
            self.inputs.int_or_vector_int.connect(int_or_vector_int)
        if location is not None:
            self.inputs.location.connect(location)

    @staticmethod
    def _spec():
        description = """take a int or a vector of int and transform it in a one entity field
            of location &quot;numeric&quot;."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="int_or_vector_int",
                    type_names=["int32", "vector<int32>"],
                    optional=False,
                    document="""Int or vector of int""",
                ),
                1: PinSpecification(
                    name="location",
                    type_names=["string"],
                    optional=True,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="scoping",
                    type_names=["scoping"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="scopingify", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsIntsToScoping
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsIntsToScoping
        """
        return super().outputs


class InputsIntsToScoping(_Inputs):
    """Intermediate class used to connect user inputs to
    ints_to_scoping operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.ints_to_scoping()
    >>> my_int_or_vector_int = int()
    >>> op.inputs.int_or_vector_int.connect(my_int_or_vector_int)
    >>> my_location = str()
    >>> op.inputs.location.connect(my_location)
    """

    def __init__(self, op: Operator):
        super().__init__(ints_to_scoping._spec().inputs, op)
        self._int_or_vector_int = Input(ints_to_scoping._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._int_or_vector_int)
        self._location = Input(ints_to_scoping._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._location)

    @property
    def int_or_vector_int(self):
        """Allows to connect int_or_vector_int input to the operator.

        Int or vector of int

        Parameters
        ----------
        my_int_or_vector_int : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.ints_to_scoping()
        >>> op.inputs.int_or_vector_int.connect(my_int_or_vector_int)
        >>> # or
        >>> op.inputs.int_or_vector_int(my_int_or_vector_int)
        """
        return self._int_or_vector_int

    @property
    def location(self):
        """Allows to connect location input to the operator.

        Parameters
        ----------
        my_location : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.ints_to_scoping()
        >>> op.inputs.location.connect(my_location)
        >>> # or
        >>> op.inputs.location(my_location)
        """
        return self._location


class OutputsIntsToScoping(_Outputs):
    """Intermediate class used to get outputs from
    ints_to_scoping operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.ints_to_scoping()
    >>> # Connect inputs : op.inputs. ...
    >>> result_scoping = op.outputs.scoping()
    """

    def __init__(self, op: Operator):
        super().__init__(ints_to_scoping._spec().outputs, op)
        self._scoping = Output(ints_to_scoping._spec().output_pin(0), 0, op)
        self._outputs.append(self._scoping)

    @property
    def scoping(self):
        """Allows to get scoping output of the operator

        Returns
        ----------
        my_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.ints_to_scoping()
        >>> # Connect inputs : op.inputs. ...
        >>> result_scoping = op.outputs.scoping()
        """  # noqa: E501
        return self._scoping



"""
forward_meshes_container
========================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class forward_meshes_container(Operator):
    """Returns the input mesh or meshes container into a meshes container.

    Parameters
    ----------
    meshes : MeshesContainer or MeshedRegion
    default_label : str, optional
        This default label is used if a new meshes
        container needs to be created
        (default is unknown)


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.forward_meshes_container()

    >>> # Make input connections
    >>> my_meshes = dpf.MeshesContainer()
    >>> op.inputs.meshes.connect(my_meshes)
    >>> my_default_label = str()
    >>> op.inputs.default_label.connect(my_default_label)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.forward_meshes_container(
    ...     meshes=my_meshes,
    ...     default_label=my_default_label,
    ... )

    >>> # Get output data
    >>> result_meshes_container = op.outputs.meshes_container()
    """

    def __init__(self, meshes=None, default_label=None, config=None, server=None):
        super().__init__(name="forward_meshes_container", config=config, server=server)
        self._inputs = InputsForwardMeshesContainer(self)
        self._outputs = OutputsForwardMeshesContainer(self)
        if meshes is not None:
            self.inputs.meshes.connect(meshes)
        if default_label is not None:
            self.inputs.default_label.connect(default_label)

    @staticmethod
    def _spec():
        description = (
            """Returns the input mesh or meshes container into a meshes container."""
        )
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="meshes",
                    type_names=["meshes_container", "abstract_meshed_region"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="default_label",
                    type_names=["string"],
                    optional=True,
                    document="""This default label is used if a new meshes
        container needs to be created
        (default is unknown)""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="meshes_container",
                    type_names=["meshes_container"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="forward_meshes_container", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsForwardMeshesContainer
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsForwardMeshesContainer
        """
        return super().outputs


class InputsForwardMeshesContainer(_Inputs):
    """Intermediate class used to connect user inputs to
    forward_meshes_container operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.forward_meshes_container()
    >>> my_meshes = dpf.MeshesContainer()
    >>> op.inputs.meshes.connect(my_meshes)
    >>> my_default_label = str()
    >>> op.inputs.default_label.connect(my_default_label)
    """

    def __init__(self, op: Operator):
        super().__init__(forward_meshes_container._spec().inputs, op)
        self._meshes = Input(forward_meshes_container._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._meshes)
        self._default_label = Input(
            forward_meshes_container._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._default_label)

    @property
    def meshes(self):
        """Allows to connect meshes input to the operator.

        Parameters
        ----------
        my_meshes : MeshesContainer or MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.forward_meshes_container()
        >>> op.inputs.meshes.connect(my_meshes)
        >>> # or
        >>> op.inputs.meshes(my_meshes)
        """
        return self._meshes

    @property
    def default_label(self):
        """Allows to connect default_label input to the operator.

        This default label is used if a new meshes
        container needs to be created
        (default is unknown)

        Parameters
        ----------
        my_default_label : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.forward_meshes_container()
        >>> op.inputs.default_label.connect(my_default_label)
        >>> # or
        >>> op.inputs.default_label(my_default_label)
        """
        return self._default_label


class OutputsForwardMeshesContainer(_Outputs):
    """Intermediate class used to get outputs from
    forward_meshes_container operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.forward_meshes_container()
    >>> # Connect inputs : op.inputs. ...
    >>> result_meshes_container = op.outputs.meshes_container()
    """

    def __init__(self, op: Operator):
        super().__init__(forward_meshes_container._spec().outputs, op)
        self._meshes_container = Output(
            forward_meshes_container._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._meshes_container)

    @property
    def meshes_container(self):
        """Allows to get meshes_container output of the operator

        Returns
        ----------
        my_meshes_container : MeshesContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.forward_meshes_container()
        >>> # Connect inputs : op.inputs. ...
        >>> result_meshes_container = op.outputs.meshes_container()
        """  # noqa: E501
        return self._meshes_container



from .beam_properties import beam_properties 
from .change_cs import change_cs 
from .combine_levelset import combine_levelset 
from .decimate_mesh import decimate_mesh 
from .exclude_levelset import exclude_levelset 
from .external_layer import external_layer 
from .from_field import from_field 
from .from_scoping import from_scoping 
from .from_scopings import from_scopings 
from .make_plane_levelset import make_plane_levelset 
from .make_sphere_levelset import make_sphere_levelset 
from .mesh_clip import mesh_clip 
from .mesh_cut import mesh_cut 
from .mesh_extraction import mesh_extraction 
from .mesh_get_attribute import mesh_get_attribute 
from .mesh_plan_clip import mesh_plan_clip 
from .mesh_provider import mesh_provider 
from .mesh_to_graphics import mesh_to_graphics 
from .mesh_to_graphics_edges import mesh_to_graphics_edges 
from .mesh_to_pyvista import mesh_to_pyvista 
from .meshes_provider import meshes_provider 
from .node_coordinates import node_coordinates 
from .points_from_coordinates import points_from_coordinates 
from .skin import skin 
from .split_fields import split_fields 
from .split_mesh import split_mesh 
from .stl_export import stl_export 
from .tri_mesh_skin import tri_mesh_skin 
from .wireframe import wireframe 



"""
mechanical_csv_to_field
=======================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class mechanical_csv_to_field(Operator):
    """Reads mechanical exported csv file

    Parameters
    ----------
    unit : Class Dataprocessing::Unit::Cunit
    mesh : MeshedRegion, optional
    data_sources : DataSources
    requested_location : str or FieldDefinition


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.serialization.mechanical_csv_to_field()

    >>> # Make input connections
    >>> my_unit = dpf.Class Dataprocessing::Unit::Cunit()
    >>> op.inputs.unit.connect(my_unit)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    >>> my_requested_location = str()
    >>> op.inputs.requested_location.connect(my_requested_location)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.serialization.mechanical_csv_to_field(
    ...     unit=my_unit,
    ...     mesh=my_mesh,
    ...     data_sources=my_data_sources,
    ...     requested_location=my_requested_location,
    ... )

    >>> # Get output data
    >>> result_field = op.outputs.field()
    """

    def __init__(
        self,
        unit=None,
        mesh=None,
        data_sources=None,
        requested_location=None,
        config=None,
        server=None,
    ):
        super().__init__(name="mechanical_csv_to_field", config=config, server=server)
        self._inputs = InputsMechanicalCsvToField(self)
        self._outputs = OutputsMechanicalCsvToField(self)
        if unit is not None:
            self.inputs.unit.connect(unit)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)
        if data_sources is not None:
            self.inputs.data_sources.connect(data_sources)
        if requested_location is not None:
            self.inputs.requested_location.connect(requested_location)

    @staticmethod
    def _spec():
        description = """Reads mechanical exported csv file"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="unit",
                    type_names=["class dataProcessing::unit::CUnit"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=True,
                    document="""""",
                ),
                4: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=False,
                    document="""""",
                ),
                9: PinSpecification(
                    name="requested_location",
                    type_names=["string", "field_definition"],
                    optional=False,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="field",
                    type_names=["field"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="mechanical_csv_to_field", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMechanicalCsvToField
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsMechanicalCsvToField
        """
        return super().outputs


class InputsMechanicalCsvToField(_Inputs):
    """Intermediate class used to connect user inputs to
    mechanical_csv_to_field operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.serialization.mechanical_csv_to_field()
    >>> my_unit = dpf.Class Dataprocessing::Unit::Cunit()
    >>> op.inputs.unit.connect(my_unit)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    >>> my_requested_location = str()
    >>> op.inputs.requested_location.connect(my_requested_location)
    """

    def __init__(self, op: Operator):
        super().__init__(mechanical_csv_to_field._spec().inputs, op)
        self._unit = Input(mechanical_csv_to_field._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._unit)
        self._mesh = Input(mechanical_csv_to_field._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._mesh)
        self._data_sources = Input(
            mechanical_csv_to_field._spec().input_pin(4), 4, op, -1
        )
        self._inputs.append(self._data_sources)
        self._requested_location = Input(
            mechanical_csv_to_field._spec().input_pin(9), 9, op, -1
        )
        self._inputs.append(self._requested_location)

    @property
    def unit(self):
        """Allows to connect unit input to the operator.

        Parameters
        ----------
        my_unit : Class Dataprocessing::Unit::Cunit

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.mechanical_csv_to_field()
        >>> op.inputs.unit.connect(my_unit)
        >>> # or
        >>> op.inputs.unit(my_unit)
        """
        return self._unit

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator.

        Parameters
        ----------
        my_mesh : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.mechanical_csv_to_field()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator.

        Parameters
        ----------
        my_data_sources : DataSources

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.mechanical_csv_to_field()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> # or
        >>> op.inputs.data_sources(my_data_sources)
        """
        return self._data_sources

    @property
    def requested_location(self):
        """Allows to connect requested_location input to the operator.

        Parameters
        ----------
        my_requested_location : str or FieldDefinition

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.mechanical_csv_to_field()
        >>> op.inputs.requested_location.connect(my_requested_location)
        >>> # or
        >>> op.inputs.requested_location(my_requested_location)
        """
        return self._requested_location


class OutputsMechanicalCsvToField(_Outputs):
    """Intermediate class used to get outputs from
    mechanical_csv_to_field operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.serialization.mechanical_csv_to_field()
    >>> # Connect inputs : op.inputs. ...
    >>> result_field = op.outputs.field()
    """

    def __init__(self, op: Operator):
        super().__init__(mechanical_csv_to_field._spec().outputs, op)
        self._field = Output(mechanical_csv_to_field._spec().output_pin(0), 0, op)
        self._outputs.append(self._field)

    @property
    def field(self):
        """Allows to get field output of the operator

        Returns
        ----------
        my_field : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.mechanical_csv_to_field()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field = op.outputs.field()
        """  # noqa: E501
        return self._field



"""
elemental_nodal_to_nodal
========================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class elemental_nodal_to_nodal(Operator):
    """Transforms an Elemental Nodal field into a Nodal field using an
    averaging process. The result is computed on a given node's
    scoping.

    Parameters
    ----------
    field : Field or FieldsContainer
        Field or fields container with only one field
        is expected
    mesh_scoping : Scoping, optional
        Average only on these entities
    should_average : bool, optional
        Each nodal value is divided by the number of
        elements linked to this node (default
        is true for discrete quantities).
    extend_to_mid_nodes : bool, optional
        Compute mid nodes (when available) by
        averaging the neighbour primary
        nodes.
    extend_weights_to_mid_nodes : bool, optional
        Extends weights to mid nodes (when
        available). default is false.
    mesh : MeshedRegion, optional


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.averaging.elemental_nodal_to_nodal()

    >>> # Make input connections
    >>> my_field = dpf.Field()
    >>> op.inputs.field.connect(my_field)
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_should_average = bool()
    >>> op.inputs.should_average.connect(my_should_average)
    >>> my_extend_to_mid_nodes = bool()
    >>> op.inputs.extend_to_mid_nodes.connect(my_extend_to_mid_nodes)
    >>> my_extend_weights_to_mid_nodes = bool()
    >>> op.inputs.extend_weights_to_mid_nodes.connect(my_extend_weights_to_mid_nodes)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.averaging.elemental_nodal_to_nodal(
    ...     field=my_field,
    ...     mesh_scoping=my_mesh_scoping,
    ...     should_average=my_should_average,
    ...     extend_to_mid_nodes=my_extend_to_mid_nodes,
    ...     extend_weights_to_mid_nodes=my_extend_weights_to_mid_nodes,
    ...     mesh=my_mesh,
    ... )

    >>> # Get output data
    >>> result_field = op.outputs.field()
    >>> result_weight = op.outputs.weight()
    """

    def __init__(
        self,
        field=None,
        mesh_scoping=None,
        should_average=None,
        extend_to_mid_nodes=None,
        extend_weights_to_mid_nodes=None,
        mesh=None,
        config=None,
        server=None,
    ):
        super().__init__(name="elemental_nodal_To_nodal", config=config, server=server)
        self._inputs = InputsElementalNodalToNodal(self)
        self._outputs = OutputsElementalNodalToNodal(self)
        if field is not None:
            self.inputs.field.connect(field)
        if mesh_scoping is not None:
            self.inputs.mesh_scoping.connect(mesh_scoping)
        if should_average is not None:
            self.inputs.should_average.connect(should_average)
        if extend_to_mid_nodes is not None:
            self.inputs.extend_to_mid_nodes.connect(extend_to_mid_nodes)
        if extend_weights_to_mid_nodes is not None:
            self.inputs.extend_weights_to_mid_nodes.connect(extend_weights_to_mid_nodes)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)

    @staticmethod
    def _spec():
        description = """Transforms an Elemental Nodal field into a Nodal field using an
            averaging process. The result is computed on a given
            node's scoping."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="field",
                    type_names=["field", "fields_container"],
                    optional=False,
                    document="""Field or fields container with only one field
        is expected""",
                ),
                1: PinSpecification(
                    name="mesh_scoping",
                    type_names=["scoping"],
                    optional=True,
                    document="""Average only on these entities""",
                ),
                2: PinSpecification(
                    name="should_average",
                    type_names=["bool"],
                    optional=True,
                    document="""Each nodal value is divided by the number of
        elements linked to this node (default
        is true for discrete quantities).""",
                ),
                4: PinSpecification(
                    name="extend_to_mid_nodes",
                    type_names=["bool"],
                    optional=True,
                    document="""Compute mid nodes (when available) by
        averaging the neighbour primary
        nodes.""",
                ),
                5: PinSpecification(
                    name="extend_weights_to_mid_nodes",
                    type_names=["bool"],
                    optional=True,
                    document="""Extends weights to mid nodes (when
        available). default is false.""",
                ),
                7: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=True,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="field",
                    type_names=["field"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="weight",
                    type_names=["property_field"],
                    optional=False,
                    document="""Provides the number of times it was found in
        the elemental nodal field, for each
        node. can be used to average later.""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="elemental_nodal_To_nodal", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsElementalNodalToNodal
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsElementalNodalToNodal
        """
        return super().outputs


class InputsElementalNodalToNodal(_Inputs):
    """Intermediate class used to connect user inputs to
    elemental_nodal_to_nodal operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.averaging.elemental_nodal_to_nodal()
    >>> my_field = dpf.Field()
    >>> op.inputs.field.connect(my_field)
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_should_average = bool()
    >>> op.inputs.should_average.connect(my_should_average)
    >>> my_extend_to_mid_nodes = bool()
    >>> op.inputs.extend_to_mid_nodes.connect(my_extend_to_mid_nodes)
    >>> my_extend_weights_to_mid_nodes = bool()
    >>> op.inputs.extend_weights_to_mid_nodes.connect(my_extend_weights_to_mid_nodes)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    """

    def __init__(self, op: Operator):
        super().__init__(elemental_nodal_to_nodal._spec().inputs, op)
        self._field = Input(elemental_nodal_to_nodal._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._field)
        self._mesh_scoping = Input(
            elemental_nodal_to_nodal._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._mesh_scoping)
        self._should_average = Input(
            elemental_nodal_to_nodal._spec().input_pin(2), 2, op, -1
        )
        self._inputs.append(self._should_average)
        self._extend_to_mid_nodes = Input(
            elemental_nodal_to_nodal._spec().input_pin(4), 4, op, -1
        )
        self._inputs.append(self._extend_to_mid_nodes)
        self._extend_weights_to_mid_nodes = Input(
            elemental_nodal_to_nodal._spec().input_pin(5), 5, op, -1
        )
        self._inputs.append(self._extend_weights_to_mid_nodes)
        self._mesh = Input(elemental_nodal_to_nodal._spec().input_pin(7), 7, op, -1)
        self._inputs.append(self._mesh)

    @property
    def field(self):
        """Allows to connect field input to the operator.

        Field or fields container with only one field
        is expected

        Parameters
        ----------
        my_field : Field or FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.averaging.elemental_nodal_to_nodal()
        >>> op.inputs.field.connect(my_field)
        >>> # or
        >>> op.inputs.field(my_field)
        """
        return self._field

    @property
    def mesh_scoping(self):
        """Allows to connect mesh_scoping input to the operator.

        Average only on these entities

        Parameters
        ----------
        my_mesh_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.averaging.elemental_nodal_to_nodal()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> # or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)
        """
        return self._mesh_scoping

    @property
    def should_average(self):
        """Allows to connect should_average input to the operator.

        Each nodal value is divided by the number of
        elements linked to this node (default
        is true for discrete quantities).

        Parameters
        ----------
        my_should_average : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.averaging.elemental_nodal_to_nodal()
        >>> op.inputs.should_average.connect(my_should_average)
        >>> # or
        >>> op.inputs.should_average(my_should_average)
        """
        return self._should_average

    @property
    def extend_to_mid_nodes(self):
        """Allows to connect extend_to_mid_nodes input to the operator.

        Compute mid nodes (when available) by
        averaging the neighbour primary
        nodes.

        Parameters
        ----------
        my_extend_to_mid_nodes : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.averaging.elemental_nodal_to_nodal()
        >>> op.inputs.extend_to_mid_nodes.connect(my_extend_to_mid_nodes)
        >>> # or
        >>> op.inputs.extend_to_mid_nodes(my_extend_to_mid_nodes)
        """
        return self._extend_to_mid_nodes

    @property
    def extend_weights_to_mid_nodes(self):
        """Allows to connect extend_weights_to_mid_nodes input to the operator.

        Extends weights to mid nodes (when
        available). default is false.

        Parameters
        ----------
        my_extend_weights_to_mid_nodes : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.averaging.elemental_nodal_to_nodal()
        >>> op.inputs.extend_weights_to_mid_nodes.connect(my_extend_weights_to_mid_nodes)
        >>> # or
        >>> op.inputs.extend_weights_to_mid_nodes(my_extend_weights_to_mid_nodes)
        """
        return self._extend_weights_to_mid_nodes

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator.

        Parameters
        ----------
        my_mesh : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.averaging.elemental_nodal_to_nodal()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh


class OutputsElementalNodalToNodal(_Outputs):
    """Intermediate class used to get outputs from
    elemental_nodal_to_nodal operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.averaging.elemental_nodal_to_nodal()
    >>> # Connect inputs : op.inputs. ...
    >>> result_field = op.outputs.field()
    >>> result_weight = op.outputs.weight()
    """

    def __init__(self, op: Operator):
        super().__init__(elemental_nodal_to_nodal._spec().outputs, op)
        self._field = Output(elemental_nodal_to_nodal._spec().output_pin(0), 0, op)
        self._outputs.append(self._field)
        self._weight = Output(elemental_nodal_to_nodal._spec().output_pin(1), 1, op)
        self._outputs.append(self._weight)

    @property
    def field(self):
        """Allows to get field output of the operator

        Returns
        ----------
        my_field : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.averaging.elemental_nodal_to_nodal()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field = op.outputs.field()
        """  # noqa: E501
        return self._field

    @property
    def weight(self):
        """Allows to get weight output of the operator

        Returns
        ----------
        my_weight : PropertyField

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.averaging.elemental_nodal_to_nodal()
        >>> # Connect inputs : op.inputs. ...
        >>> result_weight = op.outputs.weight()
        """  # noqa: E501
        return self._weight
